@@ -15,6 +15,7 @@ static double detectCpuTemp(const FFstrbuf* cpuName)
             case 1: error = ffDetectSmcTemps(FF_TEMP_CPU_M1X, &result); break;
             case 2: error = ffDetectSmcTemps(FF_TEMP_CPU_M2X, &result); break;
             case 3: error = ffDetectSmcTemps(FF_TEMP_CPU_M3X, &result); break;
+            case 4: error = ffDetectSmcTemps(FF_TEMP_CPU_M4X, &result); break;
             default: error = "Unsupported Apple Silicon CPU";
         }
     }
@@ -38,7 +39,7 @@ static const char* detectFrequency(FFCPUResult* cpu)
 
     FF_IOOBJECT_AUTO_RELEASE io_registry_entry_t entryDevice = IOServiceGetMatchingService(MACH_PORT_NULL, IOServiceNameMatching("pmgr"));
     if (!entryDevice)
-        return "IOServiceGetMatchingServices() failed";
+        return "IOServiceGetMatchingService() failed";
 
     if (!IOObjectConformsTo(entryDevice, "AppleARMIODevice"))
         return "\"pmgr\" should conform to \"AppleARMIODevice\"";
@@ -59,7 +60,12 @@ static const char* detectFrequency(FFCPUResult* cpu)
         pMax = pMax > pStart[i] ? pMax : pStart[i];
 
     if (pMax > 0)
-        cpu->frequencyMax = pMax / 1000 / 1000;
+    {
+        if (pMax > 100000000) // Assume that pMax is in Hz, M1~M3
+            cpu->frequencyMax = pMax / 1000 / 1000;
+        else // Assume that pMax is in kHz, M4 and later (#1394)
+            cpu->frequencyMax = pMax / 1000;
+    }
 
     return NULL;
 }
@@ -104,6 +110,7 @@ const char* ffDetectCPUImpl(const FFCPUOptions* options, FFCPUResult* cpu)
         return "sysctlbyname(machdep.cpu.brand_string) failed";
 
     ffSysctlGetString("machdep.cpu.vendor", &cpu->vendor);
+    cpu->packages = (uint16_t) ffSysctlGetInt("hw.packages", 1);
     if (cpu->vendor.length == 0 && ffStrbufStartsWithS(&cpu->name, "Apple "))
         ffStrbufAppendS(&cpu->vendor, "Apple");
 
@@ -119,6 +126,7 @@ const char* ffDetectCPUImpl(const FFCPUOptions* options, FFCPUResult* cpu)
     if(cpu->coresOnline == 1)
         cpu->coresOnline = (uint16_t) ffSysctlGetInt("hw.activecpu", 1);
 
+    ffCPUDetectByCpuid(cpu);
     detectFrequency(cpu);
     if (options->showPeCoreCount) detectCoreCount(cpu);
 
