@@ -1,5 +1,6 @@
 #include "common/printing.h"
 #include "common/jsonconfig.h"
+#include "common/size.h"
 #include "detection/displayserver/displayserver.h"
 #include "modules/display/display.h"
 #include "util/stringUtils.h"
@@ -16,14 +17,14 @@ static int sortByNameDesc(FFDisplayResult* a, FFDisplayResult* b)
     return -ffStrbufComp(&a->name, &b->name);
 }
 
-void ffPrintDisplay(FFDisplayOptions* options)
+bool ffPrintDisplay(FFDisplayOptions* options)
 {
     const FFDisplayServerResult* dsResult = ffConnectDisplayServer();
 
     if(dsResult->displays.length == 0)
     {
         ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Couldn't detect display");
-        return;
+        return false;
     }
 
     if (options->order != FF_DISPLAY_ORDER_NONE)
@@ -51,10 +52,11 @@ void ffPrintDisplay(FFDisplayOptions* options)
             {
                 if (result->refreshRate > 0)
                 {
+                    const char* space = instance.config.display.freqSpaceBeforeUnit == FF_SPACE_BEFORE_UNIT_ALWAYS ? " " : "";
                     if (options->preciseRefreshRate)
-                        ffStrbufAppendF(&buffer, " @ %gHz", result->refreshRate);
+                        ffStrbufAppendF(&buffer, " @ %g%sHz", result->refreshRate, space);
                     else
-                        ffStrbufAppendF(&buffer, " @ %iHz", (uint32_t) (result->refreshRate + 0.5));
+                        ffStrbufAppendF(&buffer, " @ %i%sHz", (uint32_t) (result->refreshRate + 0.5), space);
                 }
                 ffStrbufAppendS(&buffer, ", ");
             }
@@ -66,7 +68,7 @@ void ffPrintDisplay(FFDisplayOptions* options)
         ffStrbufTrimRight(&buffer, ' ');
         ffStrbufTrimRight(&buffer, ',');
         ffStrbufPutTo(&buffer, stdout);
-        return;
+        return true;
     }
 
     FF_STRBUF_AUTO_DESTROY key = ffStrbufCreate();
@@ -99,6 +101,7 @@ void ffPrintDisplay(FFDisplayOptions* options)
 
         FF_STRBUF_AUTO_DESTROY buffer = ffStrbufCreate();
         double inch = sqrt(result->physicalWidth * result->physicalWidth + result->physicalHeight * result->physicalHeight) / 25.4;
+        double scaleFactor = (double) result->height / (double) result->scaledHeight;
 
         if(options->moduleArgs.outputFormat.length == 0)
         {
@@ -106,22 +109,28 @@ void ffPrintDisplay(FFDisplayOptions* options)
 
             ffStrbufAppendF(&buffer, "%ix%i", result->width, result->height);
 
-            if(result->refreshRate > 0)
-            {
-                if(options->preciseRefreshRate)
-                    ffStrbufAppendF(&buffer, " @ %g Hz", ((int) (result->refreshRate * 1000 + 0.5)) / 1000.0);
-                else
-                    ffStrbufAppendF(&buffer, " @ %i Hz", (uint32_t) (result->refreshRate + 0.5));
-            }
-
             if(
                 result->scaledWidth > 0 && result->scaledWidth != result->width &&
                 result->scaledHeight > 0 && result->scaledHeight != result->height)
-                ffStrbufAppendF(&buffer, " (as %ix%i)", result->scaledWidth, result->scaledHeight);
+            {
+                ffStrbufAppendS(&buffer, " @ ");
+                ffStrbufAppendDouble(&buffer, scaleFactor, instance.config.display.fractionNdigits, instance.config.display.fractionTrailingZeros == FF_FRACTION_TRAILING_ZEROS_TYPE_ALWAYS);
+                ffStrbufAppendC(&buffer, 'x');
+            }
 
             if (inch > 1)
                 ffStrbufAppendF(&buffer, " in %i\"", (uint32_t) (inch + 0.5));
 
+            if(result->refreshRate > 0)
+            {
+                ffStrbufAppendS(&buffer, ", ");
+                if(options->preciseRefreshRate)
+                    ffStrbufAppendDouble(&buffer, result->refreshRate, 3, false);
+                else
+                    ffStrbufAppendSInt(&buffer, (int) (result->refreshRate + 0.5));
+                ffStrbufAppendS(&buffer, instance.config.display.freqSpaceBeforeUnit == FF_SPACE_BEFORE_UNIT_NEVER ? "Hz" : " Hz");
+            }
+
             bool flag = false;
             if (result->type != FF_DISPLAY_TYPE_UNKNOWN)
             {
@@ -182,8 +191,6 @@ void ffPrintDisplay(FFDisplayOptions* options)
             else
                 buf[0] = '\0';
 
-            double scaleFactor = (double) result->height / (double) result->scaledHeight;
-
             FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, ((FFformatarg[]) {
                 FF_FORMAT_ARG(result->width, "width"),
                 FF_FORMAT_ARG(result->height, "height"),
@@ -212,146 +219,120 @@ void ffPrintDisplay(FFDisplayOptions* options)
             }));
         }
     }
-}
 
-bool ffParseDisplayCommandOptions(FFDisplayOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_DISPLAY_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    if (ffStrEqualsIgnCase(subKey, "compact-type"))
-    {
-        options->compactType = (FFDisplayCompactType) ffOptionParseEnum(key, value, (FFKeyValuePair[]) {
-            { "none", FF_DISPLAY_COMPACT_TYPE_NONE },
-            { "original", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT },
-            { "scaled", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT },
-            { "original-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-            { "scaled-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-            {},
-        });
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "precise-refresh-rate"))
-    {
-        options->preciseRefreshRate = ffOptionParseBoolean(value);
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "order"))
-    {
-        options->order = (FFDisplayOrder) ffOptionParseEnum(key, value, (FFKeyValuePair[]) {
-            { "asc", FF_DISPLAY_ORDER_ASC },
-            { "desc", FF_DISPLAY_ORDER_DESC },
-            { "none", FF_DISPLAY_ORDER_NONE },
-            {},
-        });
-        return true;
-    }
-
-    return false;
+    return true;
 }
 
 void ffParseDisplayJsonObject(FFDisplayOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type") || ffStrEqualsIgnCase(key, "condition"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        if (ffStrEqualsIgnCase(key, "compactType"))
+        if (unsafe_yyjson_equals_str(key, "compactType"))
         {
-            int value;
-            const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
-                { "none", FF_DISPLAY_COMPACT_TYPE_NONE },
-                { "original", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT },
-                { "scaled", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT },
-                { "original-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-                { "scaled-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-                {},
-            });
-            if (error)
-                ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", key, error);
+            if (yyjson_is_null(val))
+                options->compactType = FF_DISPLAY_COMPACT_TYPE_NONE;
             else
-                options->compactType = (FFDisplayCompactType) value;
+            {
+                int value;
+                const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
+                    { "none", FF_DISPLAY_COMPACT_TYPE_NONE },
+                    { "original", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT },
+                    { "scaled", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT },
+                    { "original-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
+                    { "scaled-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
+                    {},
+                });
+                if (error)
+                    ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", unsafe_yyjson_get_str(key), error);
+                else
+                    options->compactType = (FFDisplayCompactType) value;
+            }
             continue;
         }
 
-        if (ffStrEqualsIgnCase(key, "preciseRefreshRate"))
+        if (unsafe_yyjson_equals_str(key, "preciseRefreshRate"))
         {
             options->preciseRefreshRate = yyjson_get_bool(val);
             continue;
         }
 
-        if (ffStrEqualsIgnCase(key, "order"))
+        if (unsafe_yyjson_equals_str(key, "order"))
         {
-            int value;
-            const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
-                { "asc", FF_DISPLAY_ORDER_ASC },
-                { "desc", FF_DISPLAY_ORDER_DESC },
-                { "none", FF_DISPLAY_ORDER_NONE },
-                {},
-            });
-            if (error)
-                ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", key, error);
+            if (yyjson_is_null(val))
+                options->order = FF_DISPLAY_ORDER_NONE;
             else
-                options->order = (FFDisplayOrder) value;
+            {
+                int value;
+                const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
+                    { "asc", FF_DISPLAY_ORDER_ASC },
+                    { "desc", FF_DISPLAY_ORDER_DESC },
+                    { "none", FF_DISPLAY_ORDER_NONE },
+                    {},
+                });
+                if (error)
+                    ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", unsafe_yyjson_get_str(key), error);
+                else
+                    options->order = (FFDisplayOrder) value;
+            }
             continue;
         }
 
-        ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateDisplayJsonConfig(FFDisplayOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyDisplayOptions))) FFDisplayOptions defaultOptions;
-    ffInitDisplayOptions(&defaultOptions);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
-
-    if (options->compactType != defaultOptions.compactType)
+    switch ((int) options->compactType)
     {
-        switch ((int) options->compactType)
-        {
-            case FF_DISPLAY_COMPACT_TYPE_NONE:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "none");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "original");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "scaled");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "original-with-refresh-rate");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "scaled-with-refresh-rate");
-                break;
-        }
+        case FF_DISPLAY_COMPACT_TYPE_NONE:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "none");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "original");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "scaled");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "original-with-refresh-rate");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "scaled-with-refresh-rate");
+            break;
     }
 
-    if (options->preciseRefreshRate != defaultOptions.preciseRefreshRate)
-        yyjson_mut_obj_add_bool(doc, module, "preciseRefreshRate", options->preciseRefreshRate);
+    yyjson_mut_obj_add_bool(doc, module, "preciseRefreshRate", options->preciseRefreshRate);
+
+    switch (options->order)
+    {
+        case FF_DISPLAY_ORDER_NONE:
+            yyjson_mut_obj_add_null(doc, module, "order");
+            break;
+        case FF_DISPLAY_ORDER_ASC:
+            yyjson_mut_obj_add_str(doc, module, "order", "asc");
+            break;
+        case FF_DISPLAY_ORDER_DESC:
+            yyjson_mut_obj_add_str(doc, module, "order", "desc");
+            break;
+    }
 }
 
-void ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
+bool ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
     const FFDisplayServerResult* dsResult = ffConnectDisplayServer();
 
     if(dsResult->displays.length == 0)
     {
         yyjson_mut_obj_add_str(doc, module, "error", "Couldn't detect display");
-        return;
+        return false;
     }
 
     yyjson_mut_val* arr = yyjson_mut_obj_add_arr(doc, module, "result");
@@ -367,6 +348,21 @@ void ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjs
         yyjson_mut_obj_add_uint(doc, output, "height", item->height);
         yyjson_mut_obj_add_real(doc, output, "refreshRate", item->refreshRate);
 
+        if (item->drrStatus == FF_DISPLAY_DRR_STATUS_UNKNOWN)
+            yyjson_mut_obj_add_null(doc, output, "drrStatus");
+        else switch (item->drrStatus)
+        {
+            case FF_DISPLAY_DRR_STATUS_DISABLED:
+                yyjson_mut_obj_add_str(doc, output, "drrStatus", "Disabled");
+                break;
+            case FF_DISPLAY_DRR_STATUS_ENABLED:
+                yyjson_mut_obj_add_str(doc, output, "drrStatus", "Enabled");
+                break;
+            default:
+                yyjson_mut_obj_add_str(doc, output, "drrStatus", "Unknown");
+                break;
+        }
+
         yyjson_mut_val* scaled = yyjson_mut_obj_add_obj(doc, obj, "scaled");
         yyjson_mut_obj_add_uint(doc, scaled, "width", item->scaledWidth);
         yyjson_mut_obj_add_uint(doc, scaled, "height", item->scaledHeight);
@@ -431,12 +427,27 @@ void ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjs
 
         yyjson_mut_obj_add_str(doc, obj, "platformApi", item->platformApi);
     }
+
+    return true;
 }
 
-static FFModuleBaseInfo ffModuleInfo = {
+void ffInitDisplayOptions(FFDisplayOptions* options)
+{
+    ffOptionInitModuleArg(&options->moduleArgs, "󰍹");
+    options->compactType = FF_DISPLAY_COMPACT_TYPE_NONE;
+    options->preciseRefreshRate = false;
+}
+
+void ffDestroyDisplayOptions(FFDisplayOptions* options)
+{
+    ffOptionDestroyModuleArg(&options->moduleArgs);
+}
+
+FFModuleBaseInfo ffDisplayModuleInfo = {
     .name = FF_DISPLAY_MODULE_NAME,
     .description = "Print resolutions, refresh rates, etc",
-    .parseCommandOptions = (void*) ffParseDisplayCommandOptions,
+    .initOptions = (void*) ffInitDisplayOptions,
+    .destroyOptions = (void*) ffDestroyDisplayOptions,
     .parseJsonObject = (void*) ffParseDisplayJsonObject,
     .printModule = (void*) ffPrintDisplay,
     .generateJsonResult = (void*) ffGenerateDisplayJsonResult,
@@ -468,16 +479,3 @@ static FFModuleBaseInfo ffModuleInfo = {
         {"Screen preferred refresh rate (in Hz)", "preferred-refresh-rate"},
     }))
 };
-
-void ffInitDisplayOptions(FFDisplayOptions* options)
-{
-    options->moduleInfo = ffModuleInfo;
-    ffOptionInitModuleArg(&options->moduleArgs, "󰍹");
-    options->compactType = FF_DISPLAY_COMPACT_TYPE_NONE;
-    options->preciseRefreshRate = false;
-}
-
-void ffDestroyDisplayOptions(FFDisplayOptions* options)
-{
-    ffOptionDestroyModuleArg(&options->moduleArgs);
-}
